function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function useArrFunc() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var _useState = React.useState([]),
      results = _useState[0],
      setResults = _useState[1];

  React.useEffect(function () {
    if (Array.isArray(args[0])) {
      setResults(args[0]);
    } else if (typeof args[0] === 'function') {
      var func = args.splice(0, 1);
      setResults(func[0].apply(func, args));
    }
  }, [JSON.stringify(args)]);
  return results;
}
function useFunction() {
  for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  var _useState2 = React.useState(),
      results = _useState2[0],
      setResults = _useState2[1];

  React.useEffect(function () {
    if (typeof args[0] === 'function') {
      var func = args.splice(0, 1);
      setResults(func[0].apply(func, args));
    } else {
      throw new Error('useFunction takes first argument as a function');
    }
  }, [JSON.stringify(args)]);
  return results;
}

var styles = {"row_cell":"_3TtXw","row_cell_invisible":"_2fzm0","row":"_32f2G","row_invisible":"_kbpmo","tag_content":"_2nfh4"};

function makeId(length) {
  if (length === void 0) {
    length = 15;
  }

  var text = '';
  var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

  for (var i = 0; i < length; i++) {
    text += possible.charAt(Math.floor(Math.random() * possible.length));
  }

  return text;
}
var isBetween = function isBetween(min, max, num) {
  var states = num >= min && max >= num;
  return states;
};
function getPosition(row, column, dimension, rowTitleWidth, columnTitleHeight) {
  if (rowTitleWidth === void 0) {
    rowTitleWidth = 0;
  }

  if (columnTitleHeight === void 0) {
    columnTitleHeight = 0;
  }

  return {
    left: rowTitleWidth + column * dimension,
    top: row * dimension + columnTitleHeight
  };
}
function getColumnCount(dimension, width, rowTitleWidth) {
  return Math.floor((width - rowTitleWidth) / dimension);
}
function getRowCount(dimension, height) {
  return Math.floor(height / dimension);
}
function resizeBars(bars, newLocation, resolver) {
  return bars.map(function (bar) {
    if (bar.editing) {
      var nBar = evaluatePosition(bar, newLocation.cell);

      if (typeof resolver === 'function') {
        nBar = resolver(nBar);
      }

      return nBar;
    }

    return bar;
  });
}
function finishEditingBars(bars) {
  return bars.map(function (bar) {
    if (bar.editing) {
      return _extends({}, bar, {
        editing: false
      });
    }

    return bar;
  });
}
function evaluatePosition(bar, newLocation) {
  if (bar.column > newLocation.column || bar.stick === 'right' && bar.length > 1) {
    bar.stick = 'right';
    var locationForRight = {
      row: bar.row,
      column: newLocation.column,
      length: bar.column - newLocation.column + bar.length
    };
    return _extends({}, bar, locationForRight);
  }

  bar.stick = 'left';
  var locationForLeft = {
    row: bar.row,
    column: bar.column,
    length: newLocation.column - bar.column + 1
  };
  return _extends({}, bar, locationForLeft);
}

function checkCollisions(bars) {
  var nBars = [].concat(bars);
  var editingBars = bars.filter(function (b) {
    return b.editing;
  });
  editingBars.forEach(function (bar) {
    var _reviewBars = reviewBars(nBars, bar),
        cBars = _reviewBars[0],
        eBar = _reviewBars[1];

    cBars.push(eBar);
    nBars = cBars;
  });
  return nBars;
}
function reviewBars(bars, eBar) {
  var editingBar = _extends({}, eBar);

  var oBars = bars.flatMap(function (b) {
    if (b.id === eBar.id) {
      return [];
    }

    if (editingBar.row === b.row) {
      var bStart = b.column + 1;
      var barStart = editingBar.column + 1;
      var bTotal = b.column + b.length;
      var editingBarTotal = editingBar.column + editingBar.length;

      if (isBetween(bStart, bTotal, barStart) || isBetween(bStart, bTotal, editingBarTotal) || isBetween(barStart, editingBarTotal, bTotal) || isBetween(barStart, editingBarTotal, bStart)) {
        var _collided = collided(b, editingBar),
            bar1 = _collided[0],
            bar2 = _collided[1];

        editingBar = bar2;
        return bar1;
      } else {
        var _removeCollision = removeCollision(b, editingBar),
            _bar = _removeCollision[0],
            _bar2 = _removeCollision[1];

        editingBar = _bar2;
        return _bar;
      }
    } else {
      var _removeCollision2 = removeCollision(b, editingBar),
          _bar3 = _removeCollision2[0],
          _bar4 = _removeCollision2[1];

      editingBar = _bar4;
      return _bar3;
    }
  });
  return [oBars, editingBar];
}
var collided = function collided(bar1, bar2) {
  bar1 = checkHasCollisionObject(bar1);
  bar1.collisions[bar2.id] = '';
  bar2 = checkHasCollisionObject(bar2);
  bar2.collisions[bar1.id] = '';
  return [bar1, bar2];
};
var checkHasCollisionObject = function checkHasCollisionObject(bar) {
  if (!bar.collisions) {
    bar.collisions = {};
  }

  return bar;
};
var removeCollision = function removeCollision(bar1, bar2) {
  bar1 = checkHasCollisionObject(bar1);
  delete bar1.collisions[bar2.id];
  bar2 = checkHasCollisionObject(bar2);
  delete bar2.collisions[bar1.id];
  return [bar1, bar2];
};

var actionTypes = {
  add: 'ADD',
  edit: 'EDIT',
  "delete": 'DELETE',
  addToEditing: 'ADD_TO_EDITING',
  doneEditing: 'DONE_EDITING',
  setBars: 'SET_BARS',
  setIsEditing: 'SET_IS_EDITING'
};

function reserverReducer(state, action) {
  switch (action.type) {
    case actionTypes.setBars:
      {
        return _extends({}, state, {
          bars: action.payload
        });
      }

    case actionTypes.edit:
      {
        var nBars = [].concat(state.bars);
        var bIndex = nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });
        nBars[bIndex] = action.payload;
        return _extends({}, state, {
          bars: nBars
        });
      }

    case actionTypes.add:
      {
        var bars = [].concat(state.bars);
        bars.push(action.payload);
        return _extends({}, state, {
          bars: bars
        });
      }

    case actionTypes["delete"]:
      {
        var _nBars = [].concat(state.bars);

        var _bIndex = _nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });

        _nBars.splice(_bIndex, 1);

        return _extends({}, state, {
          bars: _nBars
        });
      }

    case actionTypes.setIsEditing:
      {
        return _extends({}, state, {
          isEditing: action.payload
        });
      }
  }
}

function RowTitle(props) {
  return /*#__PURE__*/React__default.createElement("div", {
    className: props.isVisible ? styles.row_cell : styles.row_cell_invisible,
    style: {
      width: props.width,
      height: props.dimension + 'px',
      overflow: 'hidden'
    }
  }, props.children);
}

function useReserver(reducer, initialState) {
  var _useReducer = React.useReducer(reducer, {
    bars: initialState,
    isEditing: false
  }),
      _useReducer$ = _useReducer[0],
      bars = _useReducer$.bars,
      isEditing = _useReducer$.isEditing,
      dispatch = _useReducer[1];

  var addBar = function addBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.add
    });
  };

  var editBar = function editBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.edit
    });
  };

  var deleteBar = function deleteBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes["delete"]
    });
  };

  var setBars = function setBars(props) {
    return dispatch({
      payload: props,
      type: actionTypes.setBars
    });
  };

  var setIsEditing = function setIsEditing(props) {
    return dispatch({
      payload: props,
      type: actionTypes.setIsEditing
    });
  };

  return {
    isEditing: isEditing,
    setIsEditing: setIsEditing,
    bars: bars,
    addBar: addBar,
    editBar: editBar,
    deleteBar: deleteBar,
    setBars: setBars
  };
}

function getContent(index, length, content, firstContent, lastContent) {
  if (index === 0) {
    return firstContent || content[index] || /*#__PURE__*/React__default.createElement("div", null);
  } else if (length - 1 === index) {
    return lastContent || content[length - 1] || /*#__PURE__*/React__default.createElement("div", null);
  }

  return content[index] || /*#__PURE__*/React__default.createElement("div", null);
}

function Bar(props) {
  return /*#__PURE__*/React__default.createElement("div", {
    role: "listitem",
    onDragStart: function onDragStart(e) {
      props.onDragStart(e, props);
    },
    onClick: function onClick(e) {
      props.onClick(e, props);
    },
    onMouseOver: function onMouseOver(e) {
      props.onMouseOver(e, props);
    },
    onContextMenu: function onContextMenu(e) {
      props.onContextMenu(e, props);
    },
    onMouseEnter: function onMouseEnter(e) {
      props.onMouseEnter(e, props);
    },
    onMouseLeave: function onMouseLeave(e) {
      props.onMouseLeave(e, props);
    },
    draggable: props.draggable,
    style: _extends({}, props.style, {
      pointerEvents: props.style.pointerEvents || 'none',
      background: props.style.background || '#0E6BA8',
      display: 'flex',
      position: 'absolute',
      zIndex: props.zIndex || '100'
    }),
    className: props.className
  }, [].concat(Array(props.length)).map(function (notUsed, i) {
    var content = getContent(i, props.length, props.content, props.firstContent, props.lastContent);
    var style = Object.assign({
      width: props.dimension,
      height: props.dimension,
      pointerEvents: props.style.pointerEvents || 'none'
    }, content.props.style || {});
    return /*#__PURE__*/React__default.createElement(React__default.Fragment, {
      key: i
    }, React__default.cloneElement(content, _extends({}, content.props, {
      style: style
    }), content.props.children));
  }), props.children);
}
Bar.defaultProps = {
  style: {},
  dimension: 20,
  onClick: function onClick() {},
  onMouseOver: function onMouseOver() {},
  onDragStart: function onDragStart() {},
  onContextMenu: function onContextMenu() {},
  onMouseEnter: function onMouseEnter() {},
  onMouseLeave: function onMouseLeave() {},
  length: 1,
  content: {}
};

function Head(props) {
  return /*#__PURE__*/React__default.createElement("div", {
    role: "columnheader",
    className: props.columnTitles.length > 0 ? styles.row : styles.row_invisible,
    style: {
      height: props.dimension
    }
  }, props.dir === 'ltr' && /*#__PURE__*/React__default.createElement(RowTitle, {
    isVisible: props.showCanton,
    width: props.rowTitleWidth,
    dimension: props.dimension
  }), props.columnTitles.map(function (headitem, i) {
    return /*#__PURE__*/React__default.createElement("div", {
      "aria-colindex": i,
      key: i,
      style: {
        width: props.dimension + 'px',
        height: props.dimension + 'px'
      },
      className: styles.row_cell
    }, headitem);
  }), props.dir === 'rtl' && /*#__PURE__*/React__default.createElement(RowTitle, {
    isVisible: props.showCanton,
    width: props.rowTitleWidth,
    dimension: props.dimension
  }));
}

function Cell(props) {
  return /*#__PURE__*/React__default.createElement("div", {
    role: "gridcell",
    "aria-colindex": props.column,
    onDragStart: function onDragStart(e) {
      e.preventDefault();
    },
    onDragOver: function onDragOver(e) {
      e.preventDefault();
      props.onDragOver({
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    className: props.className || styles.row_cell,
    onMouseEnter: function onMouseEnter(e) {
      props.onMouseEnter({
        dimension: props.dimension,
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    onMouseDown: function onMouseDown(e) {
      props.onMouseDown({
        dimension: props.dimension,
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    onMouseUp: function onMouseUp(e) {
      props.onMouseUp({
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    onDrop: function onDrop(e) {
      props.onDrop({
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    style: {
      overflow: 'hidden',
      width: props.dimension + 'px',
      height: props.dimension + 'px'
    }
  }, props.children);
}

function Peg(props) {
  return /*#__PURE__*/React__default.createElement("div", {
    style: props.style
  }, props.children);
}
Peg.defaultProps = {
  style: {
    userSelect: 'none',
    pointerEvents: 'none',
    alignItems: 'center',
    height: '100%',
    width: '100%',
    display: 'flex',
    fontSize: '10px',
    background: 'green'
  }
};

function Tag(props) {
  return /*#__PURE__*/React__default.createElement("span", {
    style: props.style,
    className: styles.tag_content
  }, props.children);
}

function Reserver(props) {
  var rowCount = useFunction(getRowCount, props.dimension, props.height);
  var columnCount = useFunction(getColumnCount, props.dimension, props.width, props.rowTitleWidth);
  var rowTitles = useArrFunc(props.rowTitles);
  var columnTitles = useArrFunc(props.columnTitles, columnCount);
  return /*#__PURE__*/React__default.createElement("div", {
    id: props.id,
    className: props.className,
    role: "grid",
    onMouseLeave: props.mouseLeaveGrid,
    style: _extends({}, props.style, {
      position: 'relative'
    })
  }, /*#__PURE__*/React__default.createElement(Head, {
    columnTitles: columnTitles,
    columnCount: columnCount,
    rowTitleWidth: props.rowTitleWidth,
    dimension: props.dimension,
    showCanton: rowTitles.length > 0,
    dir: props.dir
  }), [].concat(Array(rowCount)).map(function (x, r) {
    return /*#__PURE__*/React__default.createElement("div", {
      role: "rowgroup",
      className: styles.row,
      key: r,
      style: {
        height: props.dimension,
        display: 'flex'
      }
    }, props.dir === 'ltr' && /*#__PURE__*/React__default.createElement(RowTitle, {
      isVisible: rowTitles.length > 0,
      width: props.rowTitleWidth,
      dimension: props.dimension
    }, rowTitles[r]), [].concat(Array(columnCount)).map(function (x, c) {
      return /*#__PURE__*/React__default.createElement(Cell, {
        key: "r" + r + "c" + c,
        onMouseDown: props.mouseDownCell,
        onMouseEnter: props.mouseEnterCell,
        onMouseUp: props.mouseUpCell,
        onDrop: props.mouseDropCell,
        onDragOver: props.mouseDragOverCell,
        dimension: props.dimension,
        className: props.cellClassName,
        column: c,
        row: r
      }, props.content["r" + r + "c" + c]);
    }), props.dir === 'rtl' && /*#__PURE__*/React__default.createElement(RowTitle, {
      isVisible: rowTitles.length > 0,
      width: props.rowTitleWidth,
      dimension: props.dimension
    }, rowTitles[r]));
  }), /*#__PURE__*/React__default.createElement("div", {
    role: "list"
  }, typeof props.children === 'function' && props.children({
    rowCount: rowCount,
    columnCount: columnCount,
    rowTitleWidth: props.rowTitleWidth,
    dimension: props.dimension,
    columnTitleHeight: columnTitles.length > 0 ? props.dimension : 0
  }), Array.isArray(props.children) && props.children));
}

function createBar(dimension, startLocation) {
  return _extends({
    id: makeId(),
    length: 1,
    dimension: dimension,
    editing: true
  }, startLocation);
}
Reserver.defaultProps = {
  columnTitles: [],
  rowTitles: [],
  content: {},
  dimension: 20,
  width: 500,
  height: 500,
  rowTitleWidth: 0,
  dir: 'ltr',
  mouseEnterCell: function mouseEnterCell() {},
  mouseDownCell: function mouseDownCell() {},
  mouseUpCell: function mouseUpCell() {},
  mouseDragOverCell: function mouseDragOverCell() {},
  mouseDropCell: function mouseDropCell() {},
  mouseLeaveGrid: function mouseLeaveGrid() {}
};

exports.Bar = Bar;
exports.Peg = Peg;
exports.Reserver = Reserver;
exports.Tag = Tag;
exports.actionTypes = actionTypes;
exports.checkCollisions = checkCollisions;
exports.createBar = createBar;
exports.default = Reserver;
exports.evaluatePosition = evaluatePosition;
exports.finishEditingBars = finishEditingBars;
exports.getPosition = getPosition;
exports.makeId = makeId;
exports.reserverReducer = reserverReducer;
exports.resizeBars = resizeBars;
exports.useReserver = useReserver;
//# sourceMappingURL=index.js.map
